@@ -37,13 +37,17 @@ typedef struct _PangoOTBuffer  PangoOTBuffer;
 typedef struct _PangoOTGlyph   PangoOTGlyph;
 typedef struct _PangoOTRuleset PangoOTRuleset;
 
-typedef enum 
+typedef enum
 {
   PANGO_OT_TABLE_GSUB,
   PANGO_OT_TABLE_GPOS
 } PangoOTTableType;
 
-/* Note that this much match OTLGlyphItem */
+
+#define PANGO_OT_ALL_GLYPHS		((guint)0xFFFF)
+#define PANGO_OT_DEFAULT_LANGUAGE	((guint)0xFFFF)
+
+/* Note that this must match HB_GlyphItem */
 struct _PangoOTGlyph
 {
   guint    glyph;
@@ -51,6 +55,7 @@ struct _PangoOTGlyph
   guint    cluster;
   gushort  component;
   gushort  ligID;
+  gushort  property_cache;    /* Internal */
 };
 
 PangoOTInfo *pango_ot_info_get (FT_Face face);
@@ -90,7 +95,7 @@ void           pango_ot_buffer_clear      (PangoOTBuffer     *buffer);
 void           pango_ot_buffer_set_rtl    (PangoOTBuffer     *buffer,
 					   gboolean           rtl);
 void           pango_ot_buffer_add_glyph  (PangoOTBuffer     *buffer,
-					   guint              glyph_index,
+					   guint              glyph,
 					   guint              properties,
 					   guint              cluster);
 void           pango_ot_buffer_get_glyphs (PangoOTBuffer     *buffer,
